from PyQt5 import QtCore
import dns.query, dns.resolver, dns.zone, dns.rdataset, dns.rdatatype

windowHeading = 'za - DNS zone admin'

class ZoneModel(QtCore.QAbstractTableModel):
    def __init__(self, data=[[]], parent=None):
        super().__init__(parent)
        self.zone = Zone('some.do.main')

    def headerData(self, section: int, orientation: QtCore.Qt.Orientation, role: int):
        if role == QtCore.Qt.DisplayRole:
            if orientation == QtCore.Qt.Horizontal:
                return ['OwnerName', 'TTL', 'Type', 'Rdata'][section]
            else:
                return None

    def columnCount(self, parent=None):
        return self.zone.columnCount()

    def rowCount(self, parent=None):
        return self.zone.rowCount()

    def data(self, index: QtCore.QModelIndex, role: int):
        if role == QtCore.Qt.DisplayRole:
            row = index.row()
            col = index.column()
            return self.zone.data(row, col)


class Zone(object):
    IP_XFR_NS = '1234:5678:9abc:def::1'

    def __init__(self, zone_name):
        self.zone_name = zone_name
        self.z = dns.zone
        self.d = [['', '', '', '']]

    def data(self, row: int, column: int) -> str:
        if not self.d:
            self.loadZone()
        v = self.d[row][column]
        if not v: v = ''
        return str(v)

    def loadZone(self):
        row = 0
        self.z = dns.zone.from_xfr(dns.query.xfr(self.IP_XFR_NS, self.zone_name))

        for name in self.z.keys():
            name = str(name)
            if name == '@': name = ''
            self.d[row][0] = name
            node = self.z[name]
            for the_rdataset in node:
                self.d[row][1] = str(the_rdataset.ttl)
                for rdata in the_rdataset:
                    self.d[row][2] = dns.rdatatype.to_text(the_rdataset.rdtype)
                    self.d[row][3] = str(rdata)
                    row = row + 1
                    self.d.append(['', '', '', ''])


    def columnCount(self):
        if len(self.d) < 2:
            self.loadZone()
        return len(self.d[0])

    def rowCount(self):
        if len(self.d) < 2:
            self.loadZone()
        return len(self.d)
